
/* ***********************************************
 * file		: fat.c
 * 		: fat function for MP3 player
 * author	: mowin
 * date		: 2008-07-03
 * **********************************************/

#define FAT_DEBUG 1

#include <avr/io.h>

#include "../sd/sd.h"
#include "fat.h"

#if FAT_DEBUG
#include <avr/pgmspace.h>
#include "../usart/usart.h"
#endif

uint8_t (* fat_read_block)(uint32_t sector, uint8_t *block_buffer) = sd_read_block;

/* other */
uint8_t FAT32_enable = FALSE;

/* FAT global var */
uint16_t bytes_per_sector; 	/* 每扇区字节数，通常为512 */
uint8_t  sectors_per_cluster; 	/* 每簇扇区数 */

uint32_t first_fat_sector;	/* FAT表开始扇区 */
uint32_t first_data_sector;	/* 簇2第一扇区是数据区开始 */

/* FAT16 */
uint16_t first_root_sector;	/* 根目录开始扇区 */
uint16_t root_dir_sectors; 	/* 根目录占扇区数 */

/* FAT32 */
uint32_t root_cluster;		/* FAT32根目录首簇号 */

/* capacity */
uint16_t capacity;

/* get functions */
uint16_t fat_get_bytes_per_sector(void)
{
    return bytes_per_sector;
}
uint8_t fat_get_sectors_per_cluster(void)
{
    return sectors_per_cluster;
}
uint32_t fat_get_first_fat_sector(void)
{
    return first_fat_sector;
}
uint32_t fat_get_first_data_sector(void)
{
    return first_data_sector;
}
uint16_t fat_get_first_root_sector(void)
{
    return first_root_sector;
}
uint16_t fat_get_root_dir_sectors(void)
{
    return root_dir_sectors;
}
uint32_t fat_get_root_cluster(void)
{
    return root_cluster;
}
uint16_t fat_get_capacity(void)
{
    return capacity;
}

/* ***********************************************
 * function	: fat init
 * argument	: void
 * return	: void
 * **********************************************/
void fat_init(void)
{
    uint8_t block_buffer[512];/* 扇区缓存 */

    struct master_boot_record *mbr = 0; /* 主引导记录 */
    struct partition_table *pt = 0; /* 分区表 */
    struct boot_sector_710 *bs = 0; /* 启动扇区 */
    struct bpb_710 *bpb = 0; /* BPB */

    uint32_t hidden_sector; /* 隐藏扇区 */

    /* 读取MBR */
    fat_read_block(0, block_buffer);

    /* 取得分区表和隐藏扇区，隐藏扇区数据即为第一分区启动扇区位置 */
    mbr = (struct master_boot_record *)block_buffer;
    pt = (struct partition_table *)mbr->mbr_partition_table;

    hidden_sector = pt->pt_preceding_sector;

    if(pt->pt_part_type == FAT32)
	FAT32_enable = TRUE;

    /* 读取启动扇区 */
    fat_read_block(hidden_sector, block_buffer);

    /* 取得BPB */
    bs = (struct boot_sector_710 *)block_buffer;
    bpb = (struct bpb_710 *)bs->bs_bpb;

    /* 每扇区字节数 */
    bytes_per_sector = bpb->bpb_byts_per_sect;
    /* 每簇扇区数 */
    sectors_per_cluster = bpb->bpb_sect_per_clus;

    /* FAT表扇首区号 */
    first_fat_sector = hidden_sector + bpb->bpb_rsvd_sect_cnt;

    if(FAT32_enable == FALSE){
	/* FAT16根目录位置(首扇区号) */
	first_root_sector = first_fat_sector + bpb->bpb_FATsz * bpb->bpb_num_FATs;
	/* FAT16根目录长度(扇区数) */
	//root_dir_sectors = (bpb->bpb_root_ent_cnt * 32) / bpb->bpb_byts_per_sect;
	root_dir_sectors = bpb->bpb_root_ent_cnt >> 4;

	/* 数据区首第一扇区（簇二第一扇区）*/
	first_data_sector = first_root_sector + root_dir_sectors;

    } else {
	/* FAT32根目录位置 */
	root_cluster = bpb->bpb_root_cluster;

	/* 数据区首第一扇区（簇二第一扇区）*/
	first_data_sector = first_fat_sector + bpb->bpb_FAT32sz * bpb->bpb_num_FATs;
    }

    /* 大小 */
    //capacity = (bpb->bpb_tot_sect_32 * bytes_per_sector) / (1024 * 1024);
    capacity = (bpb->bpb_tot_sect_32 * bytes_per_sector) >> 20;

#if FAT_DEBUG
    put_string_f(PSTR("\nBytes per sector: "));
    put_int(bytes_per_sector);

    put_string_f(PSTR("\nSectors per cluster: "));
    put_int(sectors_per_cluster);

    put_string_f(PSTR("\nFirst fat sector: "));
    put_int(first_fat_sector);

    if(FAT32_enable == FALSE){
	put_string_f(PSTR("\nFAT16\nFirst root sector: "));
	put_int(first_root_sector);

	put_string_f(PSTR("\nRoot dir sectors: "));
	put_int(root_dir_sectors);
    }
    else{
	put_string_f(PSTR("\nFAT32\nRoot cluster: "));
	put_int(root_cluster);
    }

    put_string_f(PSTR("\nFirst data sector: "));
    put_int(first_data_sector);

    put_string_f(PSTR("\nCapacity: "));
    put_int(capacity);
    put_string_f(PSTR("MB\n"));
#endif
}

/* ***********************************************
 * function	: fat read sector
 * argument	: 扇区真实位置(簇第一扇区＋簇内偏移量)
 * return	: void
 * **********************************************/
void fat_read_sector(uint32_t sector, uint8_t *sector_buffer)
{
    fat_read_block(sector, sector_buffer);
}

/* ***********************************************
 * function	: first sector of cluster
 * argument	: cluster
 * return	: first sector
 * **********************************************/
uint32_t fat_first_sector_of_cluster(uint32_t cluster)
{
    return 
	(cluster - 2) * fat_get_sectors_per_cluster() + fat_get_first_data_sector();
}

/* ***********************************************
 * function	: fat next cluster
 * argument	: current cluster
 * return	: next cluster 
 * **********************************************/
uint32_t fat_next_cluster(uint32_t current_cluster)
{
    /* 
     * 如何计算簇号在FAT表中位置
     * 每扇区512B可以容纳256个FAT16簇号，128个FAT32簇号
     * 簇号除以256或128就可以得到簇号在那个扇区中，其
     * 余数就是簇号在扇区中的位置，取得该位置内数值即 
     * 为下一簇号
     */
    uint8_t buffer[512];
    uint32_t sector_of_fat;
    uint8_t byte_offset;

    if(FAT32_enable == FALSE){
	sector_of_fat = current_cluster / 256;
	byte_offset  = current_cluster % 256;
    } else {
	sector_of_fat = current_cluster / 128;
	byte_offset  = current_cluster % 128;
    }

    fat_read_sector(fat_get_first_fat_sector() + sector_of_fat, buffer);

    if(FAT32_enable == FALSE){
	return ((uint16_t *)buffer)[byte_offset];
    } else {
	return ((uint32_t *)buffer)[byte_offset];
    }
}

/* ***********************************************
 * function	: search entry in the FAT 
 * 		:
 * argument	: *entry_number
 * 		: 为0时表示统计文件总数
 * 		: 不为0时表示查找该编号文件项
 * 		:
 * 		: *file_entry
 * 	        : 带回找到项目的信息 
 * 	        :
 * return	: void
 * **********************************************/
void fat_search(uint8_t *entry_number, struct mp3_entry *file_entry)
{
    uint8_t buffer[512];

    uint8_t file_cnt = 0;
    struct dir_entry *de = 0;
    struct long_name_entry *lne = 0;

    uint8_t i;
    uint8_t j;
    uint16_t k;

    if(FAT32_enable == FALSE){
	/* FAT16 */
	for(j = 0; j < fat_get_root_dir_sectors(); j++){
	    /* 先读取目录区一扇区 */
	    fat_read_sector((fat_get_first_root_sector() + j), buffer);
	    /* 取得一个目录项，每个目录项长度固定32B */ 
	    for(k = 0; k < 512; k += 32){
		lne = (struct long_name_entry *)(&buffer[k]);
		/* 非空目录项 */
		if(lne->ldir_ord != 0xe5){
		    if((lne->ldir_attr & ATTR_LONG_NAME_MASK) == ATTR_LONG_NAME){
			/* 长文件名项 */
		    } else {
			/* 短文件名项 */
			lne = 0;
			de = (struct dir_entry *)(&buffer[k]);
			if((de->dir_attr & (ATTR_DIRECTORY | ATTR_VOLUME_ID)) == 0){
			    /* 找到一个文件 */
			    /* 注意短文件名全部为大写字母 */
			    if(de->dir_name[8] == 'M' && de->dir_name[9] == 'P' && de->dir_name[10] == '3'){
				/* 是MP3文件 */
				file_cnt++;

				/* 获取文件信息 */
				for(i = 0; i < 11; i++)
				    file_entry->file_name[i] = de->dir_name[i];

				file_entry->first_cluster = de->first_cluster_low;/* FAT16 */
				file_entry->file_size = de->file_size;

				/* 是否是所需文件 */
				if(*entry_number == file_cnt)
				    return;
			    }
			}
			else if((de->dir_attr & (ATTR_DIRECTORY | ATTR_VOLUME_ID)) == ATTR_DIRECTORY){
			    /* 找到一个目录 */
			}
			else {
			    /* 不理 */
			}
		    }
		}
	    }
	}
	if(*entry_number == 0)
	    *entry_number = file_cnt;
    } else {
	/* FAT32 */
    }

}
