
//lcd.c

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "../spi/spi.h"
#include "lcd.h"
#include "font_6_8.h"

uint8_t (*lcd_send_byte)(uint8_t d) = spi_access_byte;

/* ************************************
 * function	: power on reset
 * argument	: void
 * return 	: void
 * ***********************************/
void lcd_power_on_reset(void)
{
    LCD_CTRL_PORT |= (_BV(LCD_CE)|_BV(LCD_DC)|_BV(LCD_RST));
    LCD_CTRL_DDR |= (_BV(LCD_CE)|_BV(LCD_DC)|_BV(LCD_RST));

    CLRBIT(LCD_CTRL_PORT, LCD_CE);
    CLRBIT(LCD_CTRL_PORT, LCD_RST);
    _delay_us(10);
    SETBIT(LCD_CTRL_PORT, LCD_RST);
    SETBIT(LCD_CTRL_PORT, LCD_CE);
}

/* ************************************
 * function	: 写一字节数据至LCD
 * argument	: d 数据
 * 		: rs 寄存器选择
 * 		: DATA = 数据
 * 		: CMD  = 命令
 * return 	: void
 * ***********************************/
void lcd_write_byte(uint8_t d, uint8_t rs)
{
    CLRBIT(LCD_CTRL_PORT, LCD_CE);

    if(rs ==  CMD) {
	CLRBIT(LCD_CTRL_PORT, LCD_DC);
    } else {
	SETBIT(LCD_CTRL_PORT,LCD_DC);//DC=1select data register
    }
    lcd_send_byte(d);

    SETBIT(LCD_CTRL_PORT, LCD_CE);
}

/* ************************************
 * function	: 初始化
 * argument	: void
 * return 	: void
 * ***********************************/
void lcd_init(void)
{
    lcd_power_on_reset();
    lcd_write_byte(0x21,CMD);	/* 使用扩展命令方式 */
    lcd_write_byte(0x80|30,CMD);/* 设置Vop */
    lcd_write_byte(0x06,CMD); 
    lcd_write_byte(0x13,CMD);
    lcd_write_byte(0x20,CMD);
    lcd_write_byte(0x0c,CMD);
    lcd_screen_clean();
}

/* ************************************
 * function	: 清屏
 * argument	: void
 * return 	: void
 * ***********************************/
void lcd_screen_clean(void)
{
#if 0
    uint8_t n = 0;
    uint16_t i;

    lcd_set_location(0, 0);
    for(i = 0;i <= 593;i++) {
	lcd_write_byte(n,DATA);
    }
    lcd_set_location(0, 0);
#endif
    /* LCD3310点阵液晶屏
     * 48*84分辨率，即6行，每行84列
     */
    uint8_t x, y;
    for(y = 0; y < 6; y++){
	x = 0;
	lcd_set_location(x, y);
	for(; x < 84; x++)
	    lcd_write_byte(0, DATA);
    }

    lcd_set_location(0, 0);
}


/* ************************************
 * function	: 设定显示坐标
 * argument	: x = 0-63 (64 * 48)
 * 		: y = 0-5
 * return 	: void
 * ***********************************/
void lcd_set_location(uint8_t x, uint8_t y)
{
    lcd_write_byte(0x80|x, CMD);
    lcd_write_byte(0x40|y, CMD);
}


/* ************************************
 * function	: 显示一个字符
 * argument	: disp_char 单个ascii字符
 * return 	: void
 * ***********************************/
void lcd_put_char(uint8_t disp_char)
{
    uint8_t col;
    //取字模ASCII - 32
    disp_char -= 32;
    //6*8字符，共6列，每列一个字节
    for (col = 0; col < 6; col++){
	lcd_write_byte(pgm_read_byte(&(font_6_8[disp_char][col])),DATA);
	//read flash use "pgm_read_byte(&(v_name[x]))"
    }
}

/* ************************************
 * function	: 显示FLASH 字符串
 * 		: 调用方式：lcd_put_string(PSTR(string))
 * argument	: disp_string 字符串指针
 * return 	: void
 * ***********************************/
void lcd_put_string(const char *disp_string)
{
    while(pgm_read_byte(disp_string)){
	lcd_put_char(pgm_read_byte(disp_string));//显示一个字符
	disp_string++;//指向一下个字符
    }
}

/* ************************************
 * function	: 显示整数
 * argument	: -9999-9999
 * return 	: void
 * ***********************************/
void lcd_put_int(int16_t d)
{
    int16_t temp, middle;
#define FALSE 0
#define TRUE  1
    uint8_t disp_en = FALSE;

    if(d & _BV(15)){
	/* 负数 */
	temp = d - 1;
	temp = ~temp;
	lcd_put_char('-');
    } else {
	/* 正数 */
	temp = d;
    }

    middle = temp / 1000;
    if(middle){
	disp_en = TRUE;
	lcd_put_char(middle + 0x30);/* 千位 */
    }

    temp = temp - (middle * 1000);
    middle = temp / 100;
    if(middle || disp_en == TRUE){
	disp_en = TRUE;
	lcd_put_char(middle + 0x30);/* 百位 */
    }

    temp = temp - (middle * 100);
    middle = temp / 10;
    if(middle || disp_en == TRUE){
	lcd_put_char(middle + 0x30);/* 十位 */
    }

    temp = temp -(middle * 10);
    middle = temp;
    lcd_put_char(middle + 0x30);/* 个位 */
}

/* end of file */

