
//vs1003.c

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include "vs1003b.h"
#include "../spi/spi.h"

//VOLUME TABLE
PROGMEM uint16_t volume[] = {
    0x6060,0x5c5c,0x5858,0x5454,
    //	0      1      2	     3	
    0x5050,0x4c4c,0x4848,0x4444,
    //	4      5      6      7
    0x4040,0x3c3c,0x3838,0x3434,
    //  8      9      10     11
    0x3030,0x2c2c,0x2828,0x2424,
    //  12     13     14     15
    0x2020,0x1c1c,0x1818,0x1414,
    //  16     17     18     19
    0x1010,0x0c0c,0x0808,0x0404
    //  20     21     22     23
};

uint8_t (*vs1003_access_byte)(uint8_t d) = spi_access_byte;

/* ********************************************
 * function	: 端口设置
 * *******************************************/
static void vs_io_init(void)
{
    VS_CTRL_PORT |= VS_CTRL_MASK;//启动上拉或输出"1"
    VS_CTRL_DDR |= SET_OUT_MASK;//MASTER OUT
    VS_CTRL_DDR &=~ SET_IN_MASK;//MASTER IN
}


/* ********************************************
 * function	: VS1003B 初始化
 * argument	: void 
 * return	: void
 * *******************************************/
void vs1003_init(void)
{	
    //set io
    vs_io_init();

    DISABLE_CS;

    CLRBIT(VS_CTRL_PORT, VS_XRST);
    _delay_ms(20);
    SETBIT(VS_CTRL_PORT, VS_XRST);

    //设置时钟寄存器
    //XTALI = 12.288M
    //设置工作倍频3	(12.288 * 3 = 36.864M)
    //允许自动调整倍频
    //最高倍频3 + 1.5 (12.288 * (3 + 1.5) = 55.296M)
    //SCI_CLOCKF = 0x8000 + 0x1800 + 0x0000
    //DREQ == 1 可以写入数据
    while(CHK_DREQ == 0){} 
    vs_write_cmd(0x03, 0x8000 + 0x1800);

#if 0
    //写入0xac45 使音频 采样率设置为44.1KHz
    //可读取0x05 寄存器获得音频采样率值
    while(CHK_DREQ == 0){}
    vs_write_cmd(0x05, 0xac45);
#endif


    //音调
    //0b 	0000 	0000 	0000 	0000
    //		1	2	3	4
    //1 高音调节（-8~7 最高位符号位0 = off）step = 1.5dB
    //2 高音频率限制（0~15）step = 1000Hz
    //3 低音提升（0~15 0 = off）step = 1dB
    //4 低音频率限制（2~15）step = 10Hz

    //高音15K(15*1000Hz)以上衰减3dB(-2*1.5dB = 0xe*1.5dB)
    //低音100Hz(10*10Hz)以下提升3dB(3*1dB)
    //0xEF3A
    while(CHK_DREQ == 0){}
    vs_write_cmd(0x02, 0xef6a);

    //设置音量值
    //音量复位值为0 （最大音量）
    //0~254
    //0 = 最大音量
    //254 = 静音
    //while(CHK_DREQ == 0){}
    //vs_write_cmd(0x0b, pgm_read_word(&volume[13]));
}


/* ********************************************
 * function	: 发送命令
 * argument	: addr 地址(8bit)
 *  		: d    命令(16bit)
 * return	: void
 * *******************************************/
void vs_write_cmd(uint8_t addr, uint16_t d)
{
    ENABLE_CMD_CS;
    vs1003_access_byte(WRITE);	//进行写操作
    vs1003_access_byte(addr);	//写入地址
    vs1003_access_byte(d >> 8);	// 命令高字节
    vs1003_access_byte(d);	//命令低字节
    DISABLE_CS;
}



/* ********************************************
 * function	: 读取VS1003 寄存器
 * argument	: addr 地址
 * return	: command(16bit)
 * *******************************************/
uint16_t vs_read_cmd(uint8_t addr)
{	
    uint16_t temp;
    ENABLE_CMD_CS;		//命令片选
    vs1003_access_byte(READ);	//进行读操作
    vs1003_access_byte(addr);	//地址
    temp = vs1003_access_byte(0xff);//高字节先
    temp <<= 8;
    temp |= vs1003_access_byte(0xff);//低字节
    DISABLE_CS;
    return temp;
}



/* ********************************************
 * function	: 发送数据
 * 		: 注意应先给出数据片选信号再
 * 		: 调用本函数
 * argument	: one byte data
 * return	: void 
 * *******************************************/
void vs_write_data(uint8_t data)
{
    vs1003_access_byte(data);
}


/* ********************************************
 * function	: vs1003软件复位
 * argument	: void
 * return	: void
 * *******************************************/
void vs1003b_soft_reset(void)
{	
    while(CHK_DREQ == 0){}
    vs_write_cmd(0x00, 0x0804);//0x0804=复位并设置vs1003模式
}

/* ********************************************
 * function	: 读取解码时间
 * argument	: void
 * return	: decode time(秒)
 * *******************************************/
uint16_t vs_read_decode_time(void)
{
    return vs_read_cmd(0x04);
}

/* ********************************************
 * function	: 清除解码时间
 * argument	: void
 * return	: void
 * *******************************************/
void vs_clear_decode_time(void)
{
    vs_write_cmd(0x04, 0x0000);
}

//end of file
